#!/bin/bash
VER="1.0"
#####################################################################################
# ShiftDaBandwidth by Turranius  ( http://www.grandis.nu/glftpd/ )                  #
#-----------------------------------------------------------------------------------#
# This script will change the speed_limit on your site. Excellent if you wish to    #
# change how much bandwidth your site should use, and you wish to change it         #
# automatically based on time of day or something.                                  #
#                                                                                   #
# The default usage is                                                              #
# ShiftDaBandwidth.sh DownloadLimit UploadLimit (KILL)                              #
# The down and up limit are in k/sec format. Not b/sec as per default.              #
# So; ShiftDaBandwidth.sh 500 200                                                   #
# would change it so the maximum download speed is 500kb/sec and up 200kb/sec.      #
# A setting of 0 means unlimited with no restriction.                               #
# If you give KILL as the third argument, it will also kick all users on site.      #
# Example: ShiftDaBandwidth.sh 500 200 KILL                                         #
#                                                                                   #
# Once setup, you can crontab it to change the numbers as you like, specific times  #
# of the day.                                                                       #
#                                                                                   #
# Setup:                                                                            #
# Copy ShiftDaBandwidth to /glftpd/bin, or anywhere you like. Make it executable.   #
# Edit below Settings. Heres a longer explanation:                                  #
#                                                                                   #
# GLCONF= Full path to your glftpd.conf. Its usually in /etc/glftpd.conf, and that  #
#         is the default setting. Unless you changed something, it is NOT the one   #
#         in /glftpd/glftpd.conf. Thats just an example file.                       #
#                                                                                   #
# LOG=    Logfile to write to. Nice to know what it did, and when.                  #
#                                                                                   #
# TEMPFOLDER = A folder to store glftpd.conf as I am editing it. Once executed,     #
#              2 files will be in this folder. glftpd.conf.org and glftpd.conf.edt  #
#              .org is the original and .edt is the edited one.                     #
#                                                                                   #
# CHANGE= Which speed_limit in your glftpd.conf do you want me to change?           #
#         The default is \/site\/\* which translates to /site/*                     #
#         You can have multiple speed_limits in your config, but this is the only   #
#         one I will change. So per default, if you have:                           #
#         speed_limit /site/* 500000 200000                                         #
#         speed_limit /site/siteops/* 0 0                                           #
#         Then I will only change the top one with the default setting.             #
#         Remember to escape any questionable characters if you change this..       #
#         If I want to change the /site/siteops setting above, I would change it to:#
#         CHANGE="\/site\/siteops\/\*"                                              #
#                                                                                   #
# TURFTPWHO=  Yes! I edited the original ftp-who so I could more easely read the    #
#             pids to kick. (If you use my KTMF script, this is the SAME file)      #
#             Either copy the included tur-ftpwho binary to somewhere and make it   #
#             executable, OR compile the included source file.                      #
#             ( gcc -o tur-ftpwho tur-ftpwho.c ).                                   #
#             The pre compiled one is compiled on Mandrake 8.1.                     #
#             This is ONLY needed if your plan on using the KILL feature.           #
#             Note: If you run glftpd with another ipc_key, check the first line    #
#             of the tur-ftpwho.c file. Change it to match your setting.            #
#             Make sure it works by running it from shell.                          # 
#                                                                                   #
# Thats about it. Try and run ShiftDaBandwidth.sh without any arguments and see     #
# what it says. Once its working, crontab it with the numbers you want, at the time #
# of the day that you want those numbers (And possbily KILL).                       #
###-------------------------------------------------------------------------------###
# If you need to contact me, /msg Turran ( or turranius / turr|away / turr|work )   #
# on Efnet. I usually hang out in #glftpd as Turran, trying to look cool.           #
#####################################################################################
# Settings:

GLCONF="/etc/glftpd.conf"                           # glftpd.conf
LOG="/glftpd/ftp-data/logs/shiftdabandwidth.log"    # Set to "" to disable.
TEMPFOLDER="/glftpd/tmp"                            # Temp storage and backup
CHANGE="\/site\/\*"                                 # Which speed_limit to change? Read docs above.

TURFTPWHO="/glftpd/bin/tur-ftpwho"                  # Path to tur-ftpwho, if your use KILL.


###############################################################################
# No change needed below here.                                                #
###############################################################################

if [ -e "$GLCONF" ]; then
  if [ -e "$LOG" ]; then
    ALLTHERE="YES"
  else
    echo "Can not find $LOG. Create the file ( touch $LOG ) and set perms in it ( chmod 777 $LOG )."
    exit 1
  fi
else
  echo "Can not read $GLCONF. Make sure you set it correct. Its usually in /etc/glftpd.conf"
  exit 1
fi

if [ "$3" = "KILL" ]; then
  KILLTHEM="TRUE"
fi

if [ "$KILLTHEM" = "TRUE" ]; then
  if [ -e "$TURFTPWHO" ]; then
    ALLTHERE="YES"
  else
    echo "You have decided to kick all users too, but I cant find $TURFTPWHO."
    echo "Check your setup!"
    exit 1
  fi
fi
 
if [ "$CHANGE" = "" ]; then
  echo "The CHANGE parameter is empty. Set it right."
  echo "The default is \/site\/\*"
  exit 1
fi

DATENOW="$(date +%D" - "%T)"

CHANGECHECK="$( echo $CHANGE | tr -d '\\' )"

SPEEDLIMIT="$( cat $GLCONF | grep -w speed_limit )"
for checks in $SPEEDLIMIT; do
  if [ "$checks" = "$CHANGECHECK" ]; then
    FOUND="YES"
  fi
done

if [ "$FOUND" != "YES" ]; then
  SPEEDLIMIT=""
fi

if [ "$SPEEDLIMIT" = "" ]; then
  echo "Can not find any existing speed_limit $CHANGECHECK in $GLCONF"
  echo "You need to edit it and insert atleast:"
  echo "speed_limit $CHANGECHECK 0 0"
  echo "somewhere in it."
  echo "We all have our different ways of building the glftpd.conf, so"
  echo "I wont insert that line for you."
  exit 1
fi

if [ "$1" = "" -o "$2" = "" ]; then
  echo "#----------------------------------------------------------------------#"
  echo "# ShiftDaBandwidth $VER by Turranius"
  echo "# Set to change speed_limit $CHANGECHECK in $GLCONF"
  if [ "$KILLTHEM" = "TRUE" ]; then
    echo "# Also set to kick all users from the site when this is done."
  fi
  echo "# Usage: ShiftDaBandwidth.sh <Max Download Speed> <Max Upload Speed>"
  echo "# "
  
  echo "# Max speeds are in kb/sec, NOT bytes/sec (default for glftpd)."
  echo "# "
  echo "# A speed limit of 0 means unlimited speed_limit"
  echo "# Example"
  echo "# ShiftDaBandwidth.sh 500 0"
  echo "# That would set the maximum speed that users can download to 500k/sec"
  echo "# and the maximum speed that users can upload to unlimited."
  echo "# Note, Doing: 'ShiftDaBandwidth.sh 500 0 KILL' will also kick all"
  echo "# the users on site. Good if you want the effect on the limit to happen"
  echo "# right now!"
  exit 0
else
  VERIFYDN="$( expr "$1" \+ "1" )"
  if [ "$VERIFYDN" = "" ]; then
    echo "You cant set your max download speed to $1."
    echo "Only use numbers, and no friggin dots."
    exit 1
  fi
  VERIFYUP="$( expr "$2" \+ "1" )"
  if [ "$VERIFYUP" = "" ]; then
    echo "You cant set your max upload speed to $2."
    echo "Only use numbers, and no friggin dots."
    exit 1
  fi
  if [ "$1" != "0" ]; then
    DN=$1"000"
  else
    DN="0"
  fi
  if [ "$2" != "0" ]; then
    UP=$2"000"
  else
    UP="0"
  fi

  if [ "$LOG" != "" ]; then
    echo "$DATENOW : Changing speed for $CHANGECHECK in $GLCONF to $DN k/sec down, $UP k/sec up." >> $LOG
  fi
  
  cp -f $GLCONF $TEMPFOLDER/glftpd.conf.org
  sed -e "s/^speed_limit $CHANGE .*/speed_limit $CHANGE $DN $UP/" $GLCONF > $TEMPFOLDER/glftpd.conf.edt
  cp -f $TEMPFOLDER/glftpd.conf.edt $GLCONF
  DIDIT="TRUE"
fi

## Kill em too?

if [ "$KILLTHEM" = "TRUE" ]; then
  if [ "$DIDIT" = "TRUE" ]; then
    if [ "$LOG" != "" ]; then
      echo "$DATENOW : Kicking all users on site too." >> $LOG
    fi
    for pid in `$TURFTPWHO | awk -F"^" '{print $2}'`; do
      kill $pid
    done
  fi
fi

exit 0